/**
 * @file EventDispatcher.hh
 *
 * @brief  Event Dispatcher (Listener class with template of event handler)
 *
 * @author  Benjamin Isnard (benjamin.isnard@ens-lyon.fr)
 *
 * @section Licence
 *
 * Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)
 *
 * - Frederic.Desprez@ens-lyon.fr (Project Manager)
 * - Eddy.Caron@ens-lyon.fr (Technical Manager)
 * - Tech@sysfera.com (Maintainer and Technical Support)
 *
 * This software is a computer program whose purpose is to provide an
 * easy and transparent access to distributed and heterogeneous
 * platforms.
 *
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty  and the software's author,  the holder
 * of the economic rights,  and the successive licensors  have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading,  using,  modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean  that it is complicated to
 * manipulate, and  that  also therefore means  that it is reserved for
 * developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions
 * enabling the security of their systems and/or data to be ensured and,
 * more generally, to use and operate it in the same conditions as
 * regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 */


#ifndef EVENTDISPATCHER_HH_
#define EVENTDISPATCHER_HH_

#include <string>
#include <map>
#include <iostream>
#include "TypeInfo.hh"
#include "EventObserver.hh"
#ifdef WIN32
   #define DIET_API_LIB __declspec(dllexport)
#else
   #define DIET_API_LIB
#endif

namespace events {
class HandlerFunctionBase {
public:
  virtual ~HandlerFunctionBase() {
  }

  void
  exec(const EventBase *event) {
    call(event);
  }

private:
  virtual void
  call(const EventBase *) = 0;
};


template <class T, class EventT>
class DIET_API_LIB MemberFunctionHandler : public HandlerFunctionBase {
public:
  typedef void (T::*MemberFunc)(EventT *);

  MemberFunctionHandler(T * instance, MemberFunc memFn)
    : _instance(instance), _function(memFn) {
  }

  void
  call(const EventBase *event) {
    (_instance->*_function)(static_cast<EventT *>(event));
  }

private:
  T *_instance;
  MemberFunc _function;
};


class DIET_API_LIB EventDispatcher : public EventObserver {
public:
  ~EventDispatcher();

  virtual void
  handleEvent(const EventBase *);

  virtual bool
  isObserver(const EventBase *) const {
    return true;
  }

  template <class T, class EventT>
  void
  registerEventFunc(T *, void (T::*memFn) (EventT *));

private:
  typedef std::map<TypeInfo, HandlerFunctionBase *> Handlers;
  Handlers _handlers;
};


template <class T, class EventT>
void
EventDispatcher::registerEventFunc(T *obj, void (T::*memFn) (EventT *)) {
  _handlers[TypeInfo(typeid(EventT))] =
    new MemberFunctionHandler<T, EventT
                              >(obj, memFn);
}
}

#endif /* ifndef EVENTDISPATCHER_HH_ */
